import os
import typing
import warnings
from pathlib import Path
from nosqllite import document
import shutil


class Group:
    def __init__(self, file_path: str) -> None:
        self.file_path = str(os.path.abspath(file_path))
        if file_path[-1] != "/":
            self.file_path += "/"
        self.name = os.path.basename(file_path).split("/")[-1]
        self.documents: typing.Dict[str, typing.Union[document.Document, Group]] = dict()
        self.load(self.file_path)

    @staticmethod
    def new(file_path):
        """Make a new database"""
        if os.path.isdir(file_path):
            warnings.warn("there is a dir with that name")
        else:
            os.mkdir(file_path)
        return Group(file_path)

    def add_group(self, name:str):
        """ add a group (dir) """
        self.documents[name] = Group.new(self.file_path + name)
        return self.documents[name] 

    def load(self, path: str):
        """Load in all documents in database"""
        files = os.listdir(path)
        if path[-1] != "/":
            path += "/"
        for f in files:
            if ".json" in f:
                name = f.split("/")[-1].split(".json")[0]
                self.documents[name] = document.Document.load(path + f)

    def add_document(self, name: str) -> document.Document:
        """Adds new document to the database"""
        if name in self.documents:
            warnings.warn("tried to make new doc but name taken")
            if isinstance(self.documents[name], Group):
                raise LookupError("Error tried to add document with same name as groups")
            return self.documents[name]
        new_doc = document.Document(self.file_path + f"{name}.json")
        self.documents[name] = new_doc
        return self.documents[name]

    def save(self):
        """saves all documents in database"""
        for _, doc in self.documents.items():
            doc.save()

    def delete_document(self, doc_name: str):
        assert doc_name in self.documents, f"Did not find document: {doc_name}"
        self.documents[doc_name].delete()
        del self.documents[doc_name]

    def delete(self):
        """delete database"""
        for itm in self.documents.values():
            itm.delete()

        shutil.rmtree(self.file_path)


    def __getitem__(self, key: str):
        return self.documents[key]

    def __setitem__(self, key, value):
        if not isinstance(value, document.Document):
            raise ValueError("set needs to be a nosqllite.Document object")
        self.documents[key] = value

    def __iter__(self):
        for _, d in self.documents.items():
            yield d

    def __str__(self) -> str:
        return f"{self.file_path}"

    def __repr__(self) -> str:
        return f"nosqllite.group({self.file_path})"

    



import nosqllite 
import os

DB_NAME = "./tests/my-nosql-lite-db"

def get_db() -> nosqllite.Database:
    global DB_NAME
    if not os.path.exists(DB_NAME):
        db = nosqllite.Database.new(DB_NAME)
    else: 
        db = nosqllite.Database(DB_NAME)
    return db


def test_database():
    db = get_db()
    assert db.name == "my-nosql-lite-db" 



def test_database_delete():
    global DB_NAME
    db = get_db()
    db.add_document("test_1")
    db.add_document("test_2")
    db["test_1"].data = [1,2,3]
    db["test_2"].data = [1,2,3]
    db.save()
    assert os.path.isdir(DB_NAME)
    assert os.path.isfile(f"{DB_NAME}/test_1.json") 

    db.delete_document("test_1")
    assert not os.path.isfile(f"{DB_NAME}/test_1.json") 

    db.delete()
    assert not os.path.isdir(DB_NAME)
    


import os
import typing
import json
import datetime
import hashlib


class Document:
    def __init__(self, file_path: str) -> None:
        self.file_path = file_path
        self.name = os.path.basename(file_path).split("/")[-1].split(".json")[0]
        self.data: typing.Union[list, dict] = {}
        self.metadata = {}
        self.is_locked = False
        if not self.is_doc(file_path):
            self._write(self.file_path, self.set_metadata(), self.data)

        self.metadata, self.data = self._read(self.file_path)
        self.has_read = True

    @staticmethod
    def is_doc(file_path: str) -> bool:
        """is document a valid doc?"""
        if not os.path.isfile(file_path) or not file_path.endswith(".json"):
            return False
        with open(file_path) as f:
            d = json.load(f)
        if not "data" in d or not "metadata" in d:
            return False
        del d
        return True

    def save(self) -> None:
        m, _ = self._read(self.file_path)
        if m["datahash"] == self.hash(self.data):
            return
        else:
            self._write(self.file_path, self.set_metadata(), self.data)

    def delete(self) -> None:
        self.data = dict()
        self.metadata = dict()
        os.remove(self.file_path)

    @staticmethod
    def hash(data) -> str:
        dhash = hashlib.sha256()
        if isinstance(data, (dict, list)):
            encoded = json.dumps(data, sort_keys=True).encode()
        else:
            encoded = str(data).encode()
        dhash.update(encoded)
        return dhash.hexdigest()

    def type_of(self):
        return type(self.data)

    def set_metadata(self) -> dict:
        self.metadata["timestamp"] = datetime.datetime.now().timestamp()
        self.metadata["datahash"] = self.hash(self.data)
        return self.metadata

    @staticmethod
    def load(file_path):
        if not Document.is_doc(file_path):
            raise ValueError(f"Tried to load a non Doc file: {file_path}")
        return Document(file_path)

    @staticmethod
    def _read(file_path) -> typing.Tuple[dict, typing.Union[list, dict]]:
        with open(file_path) as f:
            d = json.load(f)
        return d["metadata"], d["data"]

    @staticmethod
    def _write(file_path: str, metadata: dict, data: typing.Union[list, dict]) -> None:
        d = {"metadata": metadata, "data": data}
        with open(file_path, "w", encoding="utf-8") as f:
            json.dump(d, f, ensure_ascii=False, indent=4)

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        print("debug")
        self.data[key] = value

    def __iter__(self):
        for v in self.data:
            yield v

    def __str__(self) -> str:
        return str(self.data)

    def __repr__(self) -> str:
        return f"nosqllite.Document({self.file_path})"

    def __dict__(self):
        return dict({
            "data": self.data,
        })



import os
import typing
import warnings
from pathlib import Path
from nosqllite import document
from nosqllite import group


class Database(group.Group):
    def __init__(self, file_path:str):
        super().__init__(file_path)

    @staticmethod
    def new(file_path):
        """Make a new database"""
        if os.path.isdir(file_path):
            warnings.warn("there is a dir with that name")
        else:
            os.mkdir(file_path)
        return Database(file_path)

    def __repr__(self) -> str:
        return f"nosqllite.Database({self.file_path})"




import nosqllite 
import os

DB_NAME = "./tests/my-nosql-lite-db"

def get_db() -> nosqllite.Database:
    global DB_NAME
    if not os.path.exists(DB_NAME):
        db = nosqllite.Database.new(DB_NAME)
    else: 
        db = nosqllite.Database(DB_NAME)
    return db


def test_group():
    db = get_db()
    group = db.add_group("sub")
    doc = group.add_document("sub_doc")
    assert group.name == "sub" 
    doc.data = {"key": "test value"}
    db.save()


def test_group_delete(): 
    global DB_NAME
    db = get_db()
    db.delete()
    assert not os.path.isdir(DB_NAME)




from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="nosqllite",
    version="0.1.2",
    description="A simple nosql data base",
    url="https://github.com/AxelGard/nosqllite",
    author="Axel Gard",
    author_email="axel.gard@tutanota.com",
    license="MIT",
    packages=["nosqllite"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
    ],
    extras_require={
        'dev': [
            'pytest',
            "black",
        ]
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Database",
        "Topic :: Database :: Database Engines/Servers",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
    ],
)



import nosqllite
import os 
import datetime

DB_NAME = "./tests/my-nosql-lite-db"

def get_db() -> nosqllite.Database:
    global DB_NAME
    if not os.path.exists(DB_NAME):
        db = nosqllite.Database.new(DB_NAME)
    else: 
        db = nosqllite.Database(DB_NAME)
    DB_NAME = os.path.abspath(DB_NAME)
    return db

def test_document_name():
    global DB_NAME
    db = get_db()
    doc_name = f"test-doc-{datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}"
    doc = db.add_document(doc_name)
    assert doc.name == doc_name
    assert doc.file_path == DB_NAME + "/" + doc_name + ".json"
    doc.delete()
    del doc

def test_document_data():
    global DB_NAME
    db = get_db()
    doc_name = f"test-doc-{datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}"
    doc = db.add_document(doc_name)

    doc.data["some_key"] = [1,2]
    doc["some_key"].append(3)
    assert doc.data == {"some_key":[1,2,3]}
    
    doc.delete()
    del doc



def test_document_synt():
    global DB_NAME
    db = get_db()
    doc_name = f"test-doc-{datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}"
    doc = db.add_document(doc_name)

    doc.data["some_key"] = [1,2]
    doc.save()

    _, d = nosqllite.Document._read(DB_NAME + "/" + doc_name + ".json")
    assert d == doc.data
    
    doc.delete()
    del doc


def test_document_metadata():
    global DB_NAME
    db = get_db() 
    doc_name = f"test-doc-{datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}"
    doc = db.add_document(doc_name) 
    doc.data = [1,2]
    doc.set_metadata()
    expected_metadata_keys = ["timestamp", "datahash", ] 
    
    assert list(doc.metadata.keys()) == expected_metadata_keys

    doc.delete()
    del doc


def test_document_clean_up(): 
    global DB_NAME
    db = get_db() 
    db.delete()
    assert not os.path.isdir(DB_NAME)